@@ -1,6 +1,6 @@
 /*
-    Copyright (c) 2014-2019 Christian Schoenebeck
-    
+    Copyright (c) 2014-2025 Christian Schoenebeck
+
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
 */
@@ -101,7 +101,7 @@ static int findLowestRegionPoint(std::vector<gig::Instrument*>& instruments, int
     int result = -1;
     for (uint i = 0; i < instruments.size(); ++i) {
         gig::Instrument* instr = instruments[i];
-        for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+        for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
             if (rgn->KeyRange.overlaps(searchRange)) {
                 int lowest = smallestOverlapPoint(rgn->KeyRange, searchRange);
                 if (result == -1 || lowest < result) result = lowest;
@@ -123,7 +123,7 @@ static int findFirstRegionEnd(std::vector<gig::Instrument*>& instruments, int iS
     int result = -1;
     for (uint i = 0; i < instruments.size(); ++i) {
         gig::Instrument* instr = instruments[i];
-        for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+        for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
             if (rgn->KeyRange.overlaps(searchRange)) {
                 if (result == -1 || rgn->KeyRange.high < result)
                     result = rgn->KeyRange.high;
@@ -140,7 +140,7 @@ static int findFirstRegionEnd(std::vector<gig::Instrument*>& instruments, int iS
 static std::vector<gig::Region*> getAllRegionsWhichOverlapRange(gig::Instrument* instrument, DLS::range_t range) {
     //std::cout << "All regions which overlap { " << (int)range.low << ".." << (int)range.high << " } : " << std::flush;
     std::vector<gig::Region*> v;
-    for (gig::Region* rgn = instrument->GetFirstRegion(); rgn; rgn = instrument->GetNextRegion()) {
+    for (size_t r = 0; gig::Region* rgn = instrument->GetRegionAt(r); ++r) {
         if (rgn->KeyRange.overlaps(range)) {
             v.push_back(rgn);
             //std::cout << (int)rgn->KeyRange.low << ".." << (int)rgn->KeyRange.high << ", " << std::flush;
@@ -217,7 +217,7 @@ static RegionGroups groupByRegionIntersections(std::vector<gig::Instrument*>& in
  *
  * Takes a planned new region (@a regionGroup) as argument and identifies which
  * precise dimensions would have to be created for that new region, along with
- * the amount of dimension zones and their precise individual zone sizes.
+ * the amount of dimension zones.
  *
  * @param regionGroup - planned new region for a new instrument
  * @returns set of dimensions that shall be created for the given planned region
@@ -225,6 +225,9 @@ static RegionGroups groupByRegionIntersections(std::vector<gig::Instrument*>& in
 static Dimensions getDimensionsForRegionGroup(RegionGroup& regionGroup) {
     std::map<gig::dimension_t, std::set<int> > dimUpperLimits;
 
+    #if DEBUG_COMBINE_INSTRUMENTS
+    printf("dimUpperLimits = {\n");
+    #endif
     // collect all dimension region zones' upper limits
     for (RegionGroup::iterator it = regionGroup.begin();
          it != regionGroup.end(); ++it)
@@ -233,34 +236,54 @@ static Dimensions getDimensionsForRegionGroup(RegionGroup& regionGroup) {
         int previousBits = 0;
         for (uint d = 0; d < rgn->Dimensions; ++d) {
             const gig::dimension_def_t& def = rgn->pDimensionDefinitions[d];
+            #if DEBUG_COMBINE_INSTRUMENTS
+            printf("\t[rgn=%p,dim=%#x] = {", rgn, def.dimension);
+            #endif
             for (uint z = 0; z < def.zones; ++z) {
+/*
                 int dr = z << previousBits;
                 gig::DimensionRegion* dimRgn = rgn->pDimensionRegions[dr];
-                // Store the individual dimension zone sizes (or actually their
-                // upper limits here) for each dimension.
-                // HACK: Note that the velocity dimension is specially handled
-                // here. Instead of taking over custom velocity split sizes
-                // here, only a bogus number (zone index number) is stored for
-                // each velocity zone, that way only the maxiumum amount of
-                // velocity splits of all regions is stored here, and when their
-                // individual DimensionRegions are finally copied (later), the
-                // individual velocity split size are copied by that.
-                dimUpperLimits[def.dimension].insert(
+ */
+                // NOTE: Originally this function collected dimensions' upper
+                // limits. However that caused combined instruments (e.g. with
+                // unequal dimension zone counts, not being a power of two) to
+                // end up having too many dimension zones and those extra zones
+                // containing no sample. For that reason we simply collect the
+                // required amount of output dimension zones here now instead.
+                const int upperLimit =
+/*
                     (def.dimension == gig::dimension_velocity) ?
                         z : (def.split_type == gig::split_type_bit) ?
-                            ((z+1) * 128/def.zones - 1) : dimRgn->DimensionUpperLimits[dr]
-                );
+                            ((z+1) * 128/def.zones - 1) : dimRgn->DimensionUpperLimits[dr];
+ */
+                    z;
+                #if DEBUG_COMBINE_INSTRUMENTS
+                printf(" %d,", upperLimit);
+                #endif
+                dimUpperLimits[def.dimension].insert(upperLimit);
             }
             previousBits += def.bits;
+            #if DEBUG_COMBINE_INSTRUMENTS
+            printf(" }\n");
+            #endif
         }
     }
+    #if DEBUG_COMBINE_INSTRUMENTS
+    printf("}\n");
+    #endif
 
     // convert upper limit set to range vector
     Dimensions dims;
+    #if DEBUG_COMBINE_INSTRUMENTS
+    printf("dims = {\n");
+    #endif
     for (std::map<gig::dimension_t, std::set<int> >::const_iterator it = dimUpperLimits.begin();
          it != dimUpperLimits.end(); ++it)
     {
         gig::dimension_t type = it->first;
+        #if DEBUG_COMBINE_INSTRUMENTS
+        printf("\t[dim=%#x] = {", type);
+        #endif
         int iLow = 0;
         for (std::set<int>::const_iterator itNums = it->second.begin();
              itNums != it->second.end(); ++itNums)
@@ -268,9 +291,18 @@ static Dimensions getDimensionsForRegionGroup(RegionGroup& regionGroup) {
             const int iUpperLimit = *itNums;
             DLS::range_t range = { uint16_t(iLow), uint16_t(iUpperLimit) };
             dims[type].push_back(range);
+            #if DEBUG_COMBINE_INSTRUMENTS
+            printf(" %d..%d,", iLow, iUpperLimit);
+            #endif
             iLow = iUpperLimit + 1;
         }
+        #if DEBUG_COMBINE_INSTRUMENTS
+        printf(" }\n");
+        #endif
     }
+    #if DEBUG_COMBINE_INSTRUMENTS
+    printf("}\n");
+    #endif
 
     return dims;
 }
@@ -287,7 +319,7 @@ static void fillDimValues(uint* values/*[8]*/, DimensionCase dimCase, gig::Regio
         else if (iDimIndex < 0) continue;
         values[iDimIndex] = it->second;
         #if DEBUG_COMBINE_INSTRUMENTS
-        printf("%x=%d, ", type, it->second);
+        printf("0x%x=%d, ", type, it->second);
         #endif
     }
     #if DEBUG_COMBINE_INSTRUMENTS
@@ -382,6 +414,23 @@ struct CopyAssignSchedEntry {
 };
 typedef std::vector<CopyAssignSchedEntry> CopyAssignSchedule;
 
+/** @brief Constrain @a dimCase according to @a rgn's dimension zones.
+ *
+ * To avoid @a dimCase being an invalid dimension region selection on @a rgn,
+ * this function automatically wraps (if required) the individual dimension case
+ * entries according to region's actual amount of dimension zones.
+ *
+ * @param dimCase - dimension case to be potentially corrected
+ * @param rgn - the region upon @a dimCase shall be corrected for
+ */
+static void wrapDimCase(DimensionCase& dimCase, gig::Region* rgn) {
+    for (auto& it : dimCase) {
+        gig::dimension_def_t* def = rgn->GetDimensionDefinition(it.first);
+        const int zones = (def) ? def->zones : 1;
+        it.second %= zones;
+    }
+}
+
 /** @brief Schedule copying DimensionRegions from source Region to target Region.
  *
  * Schedules copying the entire articulation informations (including sample
@@ -426,6 +475,13 @@ static void scheduleCopyDimensionRegions(gig::Region* outRgn, gig::Region* inRgn
         uint dstDimValues[8] = {};
         DimensionCase srcDimCase = dimCase;
         DimensionCase dstDimCase = dimCase;
+
+        // source might have less zones than destination, so to prevent output
+        // instrument ending up with NULL samples in some of the dimension
+        // zones, wrap here the source zone according to the amount of zones the
+        // source really has
+        wrapDimCase(srcDimCase, inRgn);
+
         srcDimCase[mainDim] = iSrcMainBit;
         dstDimCase[mainDim] = iDstMainBit;
 
@@ -873,21 +929,18 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
     );
     m_treeView.show();
 
-    for (int i = 0; true; ++i) {
-        gig::Instrument* instr = gig->GetInstrument(i);
-        if (!instr) break;
-
+    for (size_t i = 0; gig::Instrument* instr = gig->GetInstrument(i); ++i) {
 #if DEBUG_COMBINE_INSTRUMENTS
         {
             std::cout << "Instrument (" << i << ") '" << instr->pInfo->Name << "' Regions: " << std::flush;
-            for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+            for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
                 std::cout << rgn->KeyRange.low << ".." << rgn->KeyRange.high << ", " << std::flush;
             }
             std::cout << std::endl;
         }
         std::cout << std::endl;
 #endif
-        
+
         Glib::ustring name(gig_to_utf8(instr->pInfo->Name));
         Gtk::TreeModel::iterator iter = m_refTreeModel->append();
         Gtk::TreeModel::Row row = *iter;
@@ -904,6 +957,7 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
     // force background to retain white also on selections
     // (this also fixes a bug with GTK 2 which often causes visibility issue
     //  with the text of the selected item)
+#if GTK_MAJOR_VERSION < 3
     {
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
         Gdk::Color white;
@@ -912,13 +966,12 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
 #endif
         white.set("#ffffff");
         GtkWidget* widget = (GtkWidget*) m_iconView.gobj();
-#if GTK_MAJOR_VERSION < 3
         gtk_widget_modify_base(widget, GTK_STATE_SELECTED, white.gobj());
         gtk_widget_modify_base(widget, GTK_STATE_ACTIVE, white.gobj());
         gtk_widget_modify_bg(widget, GTK_STATE_SELECTED, white.gobj());
         gtk_widget_modify_bg(widget, GTK_STATE_ACTIVE, white.gobj());
-#endif
     }
+#endif
 
     m_labelOrder.set_text(_("Order of the instruments to be combined:"));
 
@@ -989,14 +1042,18 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
 
 void CombineInstrumentsDialog::on_order_drag_begin(const Glib::RefPtr<Gdk::DragContext>& context)
 {
+    #if DEBUG_COMBINE_INSTRUMENTS
     printf("Drag begin\n");
+    #endif
     first_call_to_drag_data_get = true;
 }
 
 void CombineInstrumentsDialog::on_order_drag_data_get(const Glib::RefPtr<Gdk::DragContext>& context,
                                                        Gtk::SelectionData& selection_data, guint, guint)
 {
+    #if DEBUG_COMBINE_INSTRUMENTS
     printf("Drag data get\n");
+    #endif
     if (!first_call_to_drag_data_get) return;
     first_call_to_drag_data_get = false;
 
@@ -1016,7 +1073,9 @@ void CombineInstrumentsDialog::on_order_drag_data_get(const Glib::RefPtr<Gdk::Dr
         printf("Drag data get: !src\n");
         return;
     }
+    #if DEBUG_COMBINE_INSTRUMENTS
     printf("src=%ld\n", (size_t)src);
+    #endif
 
     // pass the source gig::Instrument as pointer
     selection_data.set(selection_data.get_target(), 0/*unused*/, (const guchar*)&src,
@@ -1027,7 +1086,9 @@ void CombineInstrumentsDialog::on_order_drop_drag_data_received(
     const Glib::RefPtr<Gdk::DragContext>& context, int x, int y,
     const Gtk::SelectionData& selection_data, guint, guint time)
 {
+    #if DEBUG_COMBINE_INSTRUMENTS
     printf("Drag data received\n");
+    #endif
     if (!selection_data.get_data()) {
         printf("selection_data.get_data() == NULL\n");
         return;
@@ -1038,7 +1099,9 @@ void CombineInstrumentsDialog::on_order_drop_drag_data_received(
         printf("!src\n");
         return;
     }
+    #if DEBUG_COMBINE_INSTRUMENTS
     printf("src=%ld\n", (size_t)src);
+    #endif
 
     gig::Instrument* dst = NULL;
     {
@@ -1055,7 +1118,9 @@ void CombineInstrumentsDialog::on_order_drop_drag_data_received(
         return;
     }
 
+    #if DEBUG_COMBINE_INSTRUMENTS
     printf("dragdrop received src='%s' dst='%s'\n", src->pInfo->Name.c_str(), dst->pInfo->Name.c_str());
+    #endif
 
     // swap the two items
     typedef Gtk::TreeModel::Children Children;
@@ -1123,7 +1188,7 @@ void CombineInstrumentsDialog::combineSelectedInstruments() {
             Glib::ustring name = row[m_orderColumns.m_col_name];
             gig::Instrument* instrument = row[m_orderColumns.m_col_instr];
             #if DEBUG_COMBINE_INSTRUMENTS
-            printf("Selection %d. '%s' %p\n\n", (i+1), name.c_str(), instrument));
+            printf("Selection %d. '%s' %p\n\n", (i+1), name.c_str(), instrument);
             #endif
             instruments.push_back(instrument);
         }
@@ -1192,7 +1257,7 @@ void CombineInstrumentsDialog::onSelectionChanged() {
     {
         Children allOrdered = m_refOrderModel->children();
         for (Children::iterator itOrder = allOrdered.begin();
-             itOrder != allOrdered.end(); ++itOrder)
+             itOrder != allOrdered.end(); )
         {
             Gtk::TreeModel::Row rowOrder = *itOrder;
             gig::Instrument* instr = rowOrder[m_orderColumns.m_col_instr];
@@ -1204,9 +1269,11 @@ void CombineInstrumentsDialog::onSelectionChanged() {
             }
             goto removeOrderedItem;
         nextOrderedItem:
+            ++itOrder;
             continue;
         removeOrderedItem:
-            m_refOrderModel->erase(itOrder);
+            // postfix increment here to avoid iterator invalidation
+            m_refOrderModel->erase(itOrder++);
         }
     }
 
