@@ -504,6 +504,7 @@ protected:
     Gtk::ScrolledWindow m_ScrolledWindowSamples;
     Gtk::TreeView m_TreeViewSamples;
     Glib::RefPtr<SamplesTreeStore> m_refSamplesTreeModel;
+    Glib::RefPtr<Gtk::TreeModelFilter> m_refSamplesModelFilter;
 
     class ScriptsModel : public Gtk::TreeModel::ColumnRecord {
     public:
@@ -552,10 +553,15 @@ protected:
 
     VBox m_left_vbox;
     Gtk::Notebook m_TreeViewNotebook;
+
     HBox m_searchField;
     Gtk::Label m_searchLabel;
     Gtk::Entry m_searchText;
 
+    HBox m_searchFieldSamples;
+    Gtk::Label m_searchLabelSamples;
+    Gtk::Entry m_searchTextSamples;
+
     struct SampleImportItem {
         gig::Sample*  gig_sample;  // pointer to the gig::Sample to
                                    // which the sample data should be
@@ -719,6 +725,7 @@ protected:
                                  const Gtk::TreeModel::iterator& iter);
     void instr_name_changed_by_instr_props(Gtk::TreeModel::iterator& it);
     bool instrument_row_visible(const Gtk::TreeModel::const_iterator& iter);
+    bool sample_row_visible(const Gtk::TreeModel::const_iterator& iter);
     sigc::connection instrument_name_connection;
 
     void on_action_combine_instruments();
