@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014-2017 Christian Schoenebeck
+    Copyright (c) 2014-2025 Christian Schoenebeck
     
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
@@ -139,6 +139,11 @@ ScriptEditor::ScriptEditor() :
     m_stringTag->property_foreground() = "#c40c0c"; // red
     m_tagTable->add(m_stringTag);
 
+    m_patchTag = Gtk::TextBuffer::Tag::create();
+    m_patchTag->property_foreground() = "#FF4FF3"; // pink
+    m_patchTag->property_weight() = PANGO_WEIGHT_BOLD;
+    m_tagTable->add(m_patchTag);
+
     m_commentTag = Gtk::TextBuffer::Tag::create();
     m_commentTag->property_foreground() = "#9c9c9c"; // gray
     m_tagTable->add(m_commentTag);
@@ -166,6 +171,14 @@ ScriptEditor::ScriptEditor() :
     m_lineNrTag->property_foreground() = "#CCCCCC";
     m_tagTable->add(m_lineNrTag);
 
+    m_metricTag = Gtk::TextBuffer::Tag::create();
+    m_metricTag->property_foreground() = "#000000"; // black
+    m_tagTable->add(m_metricTag);
+
+    m_stdUnitTag = Gtk::TextBuffer::Tag::create();
+    m_stdUnitTag->property_foreground() = "#50BC00"; // greenish
+    m_tagTable->add(m_stdUnitTag);
+
     // create menu
 #if USE_GTKMM_BUILDER
     m_actionGroup = Gio::SimpleActionGroup::create();
@@ -369,6 +382,12 @@ ScriptEditor::ScriptEditor() :
 #if HAS_GTKMM_SHOW_ALL_CHILDREN
     show_all_children();
 #endif
+
+#if !USE_LS_SCRIPTVM
+    // make user aware about gigedit had been compiled without liblinuxsampler support
+    m_statusLabel.set_markup(_("Limited editor features (since Gigedit was compiled without liblinuxsampler support)!"));
+    m_statusImage.set(m_warningIcon);
+#endif
 }
 
 ScriptEditor::~ScriptEditor() {
@@ -391,13 +410,6 @@ int ScriptEditor::currentFontSize() const {
 
 void ScriptEditor::setFontSize(int sizePt, bool save) {
     //printf("setFontSize(%d,%d)\n", size, save);
-
-    // make sure the real size on the screen for the editor's font is consistent
-    // on all screens (which otherwise may vary between models and DPI settings)
-    const double referenceDPI = 96;
-    double dpi = Gdk::Screen::get_default()->get_resolution();
-    double sizePx = sizePt * dpi / referenceDPI;
-
 #if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
     Pango::FontDescription fdesc;
     fdesc.set_family("monospace");
@@ -406,6 +418,11 @@ void ScriptEditor::setFontSize(int sizePt, bool save) {
     if (macIsMinMac10_6())
         fdesc.set_family("Menlo");
 # endif
+    // make sure the real size on the screen for the editor's font is consistent
+    // on all screens (which otherwise may vary between models and DPI settings)
+    const double referenceDPI = 96;
+    const double dpi = Gdk::Screen::get_default()->get_resolution();
+    const double sizePx = sizePt * dpi / referenceDPI;
     fdesc.set_size(sizePx * PANGO_SCALE);
 # if GTKMM_MAJOR_VERSION < 3
     m_lineNrView.modify_font(fdesc);
@@ -591,7 +608,10 @@ void ScriptEditor::updateSyntaxHighlightingByVM() {
         const LinuxSampler::VMSourceToken& token = tokens[i];
 
         if (token.isKeyword()) {
-            applyCodeTag(m_textBuffer, token, m_keywordTag);
+            if (token.text() == "patch")
+                applyCodeTag(m_textBuffer, token, m_patchTag);
+            else
+                applyCodeTag(m_textBuffer, token, m_keywordTag);
         } else if (token.isVariableName()) {
             applyCodeTag(m_textBuffer, token, m_variableTag);
         } else if (token.isIdentifier()) {
@@ -608,6 +628,10 @@ void ScriptEditor::updateSyntaxHighlightingByVM() {
             applyCodeTag(m_textBuffer, token, m_commentTag);
         } else if (token.isPreprocessor()) {
             applyCodeTag(m_textBuffer, token, m_preprocTag);
+        } else if (token.isMetricPrefix()) {
+            applyCodeTag(m_textBuffer, token, m_metricTag);
+        } else if (token.isStdUnit()) {
+            applyCodeTag(m_textBuffer, token, m_stdUnitTag);
         } else if (token.isNewLine()) {
         }
     }
